﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace WiimoteWhiteboard
{
    public partial class ConnectionController : UserControl
    {
        public enum ConnectState { Connected, Connecting, NotFound }

        ConnectionManager ConnectionManager;
        public ConnectionController()
        {
            InitializeComponent();

            ConnectionManager = new ConnectionManager();

            ConnectionManager.Connected += new EventHandler(ConnectionManager_Connected);
            ConnectionManager.Error += new EventHandler<EventArgs<string>>(ConnectionManager_Error);
            ConnectionManager.ConnectionFailed += new EventHandler(ConnectionManager_ConnectionFailed);

            State = ConnectState.Connected;


            if (CultureInfo.CurrentCulture.Name == "zh-TW")     //add by gsyan
            {
                gbConnectivity.Text = "藍芽連線配對";
                btnConnect.Text = "連線";
                cbAutoConnect.Text = "自動連線";
            }
        }

        public EventHandler<EventArgs<ConnectState>> StateChanged;

        ConnectState _State = ConnectState.NotFound;
        public ConnectState State
        {
            get
            {
                return _State;
            }
            private set
            {
                if (_State == value)
                    return;

                _State = value;
                switch (value)
                {
                    case ConnectState.Connected:
                        pbConnecting.Visible = false;
                        lblStatus.BackColor = Color.Green;
                        lblStatus.Text = "Connected";
                        lblStatus.Text = (CultureInfo.CurrentCulture.Name == "zh-TW" ? "已連線" : "Connected"); //add by gsyan
                        break;
                    case ConnectState.Connecting:
                        lblStatus.BackColor = Color.DarkOrange;
                        pbConnecting.Visible = false;
                        //lblStatus.Text = "Searching for Wiimotes.";
                        lblStatus.Text = (CultureInfo.CurrentCulture.Name == "zh-TW" ? "搜尋 Wiimote." : "Searching for Wiimotes."); //add by gsyan
                        break;
                    case ConnectState.NotFound:
                        lblStatus.BackColor = Color.OrangeRed;
                        pbConnecting.Visible = false;
                        //lblStatus.Text = "Wiimote search failed!";
                        lblStatus.Text = (CultureInfo.CurrentCulture.Name == "zh-TW" ? "Wiimote 搜尋失敗" : "Wiimote search failed!"); //add by gsyan
                        break;
                }

                if (StateChanged != null)
                    StateChanged(this, new EventArgs<ConnectState>(value));
            }
        }


        //disable Connect button && AutoConnect checkbox
        public void disable()
        {
            btnConnect.Enabled = false;
            cbAutoConnect.Enabled = false;
            lblStatus.Visible = false;
        }


        public void Connect()
        {
            State = ConnectState.Connecting;
            ConnectionManager.ConnectWiiMotes(true);
        }

        void ConnectionManager_ConnectionFailed(object sender, EventArgs e)
        {
            BeginInvoke((MethodInvoker)delegate()
            {
                if (cbAutoConnect.Checked)
                    Connect();
                else
                    State = ConnectState.NotFound;
            });
        }

        void ConnectionManager_Error(object sender, EventArgs<string> e)
        {
            BeginInvoke((MethodInvoker)delegate()
            {
                State = ConnectState.NotFound;
                //MessageBox.Show(e.Value);     //remark by gsyan
            });
        }

        void ConnectionManager_Connected(object sender, EventArgs e)
        {
            BeginInvoke((MethodInvoker)delegate()
            {
                State = ConnectState.Connected;
            });
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            Connect();
        }

    }
}
