/*
 * Original Code by Johnny Chung Lee: (http://johnnylee.net/projects/wii/)
 * Patches included from: (http://sourceforge.net/projects/wiiwhiteboard/)
 * Uses Wiimote Library by Brian Peek (http://wiimotelib.codeplex.com/)
 * Icon by Zhebrakov Andrew (http://www.icojoy.com/articles/49/)
 * x64 Comptibility fix and code refactoring by Rambler (http://www.hiddenramblings.com)
 * Show IR dots and Right Click by gsyan (http://gsyan888.blogspot.com)
 * Released under MIT License (http://www.opensource.org/licenses/mit-license.php)
 */

using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using WiimoteLib;
using PointF = System.Drawing.PointF;
using System.Drawing.Drawing2D;
using System.Threading;
using System.Runtime.InteropServices;
using System.Globalization;

//using MRG.Controls.UI;

namespace WiimoteWhiteboard
{
    public partial class WiimoteMainForm : Form
    {
        Controller controller;
        Graphics TrackingLayout;

        //MRG.Controls.UI.LoadingCircle loadingCircle;

        public WiimoteMainForm()
        {
            InitializeComponent();

            notifyIcon1.BalloonTipText = "Click to Restore";
            notifyIcon1.BalloonTipTitle = this.Text;
            notifyIcon1.BalloonTipIcon = ToolTipIcon.Info;
            notifyIcon1.Icon = this.Icon;
            notifyIcon1.Text = this.Text;

            if (CultureInfo.CurrentCulture.Name == "zh-TW") // locale add by gsyan
            {

                //lblBattery.Text = "qq";
                groupBox1.Text = "(ƫ)";
                groupBox2.Text = "쪺I";
                groupBox4.Text = "qq";
                cbCursorControl.Text = "ƹ";
                cbAlwaysOnTop.Text = "ܩ̤Wh";
                cbMoveCursor.Text = "ƹ";
                btnCalibrate.Text = "sw\n(Wiimote A)";
                notifyIcon1.BalloonTipText = "@U}";
                //others in ConnectionController Form
            }

            pbTrackingLayout.Image = new Bitmap(pbTrackingLayout.Width, pbTrackingLayout.Height);
            TrackingLayout = Graphics.FromImage(pbTrackingLayout.Image);
            TrackingLayout.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

            CalibrationForm.CalibrationCancelled += new EventHandler(CalibrationForm_Cancelled);
            //controller = new Controller(Screen.PrimaryScreen, this);
            controller = new Controller(Screen.PrimaryScreen);
            cbCursorControl.Checked = controller.CursorEnabled;
            if (controller.CursorEnabled)
                UpdateTrackingUtilization(controller.SourceLayout);
            controller.CalibrationStateChanged += controller_CalibrationStateChanged;
            controller.StatusInfoChanged += controller_StatusInfoChanged;

            controller.ShowLoadingCircle += DrawForm_ShowLoadingCircle;     //add by gsyan
            controller.HideLoadingCircle += DrawForm_HideLoadingCircle;     //add by gsyan
            controller.ChangeBalloonTipText += WiimoteMainForm_ChangeBalloonTipText; //add by gsyan
            controller.ShowErrorMessage += WiimoteMainForm_ErrorMessage; //add by gsyan

            ConnectionControl.StateChanged += new EventHandler<EventArgs<ConnectionController.ConnectState>>(ConnectionControl_StateChanged);

            if (!ConnectionManager.IsStackCompatible())
            {

                ConnectionControl.disable();        // add by gsyan

                try
                {
                    controller.Enable();
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);
                }
            }
            else
            {
                try
                {
                    controller.Enable();
                }
                catch (Exception)
                {
                    ConnectionControl.Connect();
                }
            }


        }


        //add by gsyan
        void DrawForm_ShowLoadingCircle(object sender, EventArgs<System.Drawing.PointF> point)
        {
            System.Drawing.PointF p = point.Value;
            BeginInvoke((MethodInvoker)delegate() { DrawForm.ShowDraw(p); });
        }
        void DrawForm_HideLoadingCircle(object sender, EventArgs e)
        {
            BeginInvoke((MethodInvoker)delegate() { DrawForm.HideDraw(); });
        }

        //add by gsyan
        void WiimoteMainForm_ChangeBalloonTipText(object sender, EventArgs<int> status)
        {
            switch (status.Value)
            {
                case 1:     //Next Click : Right Click 
                    this.notifyIcon1.BalloonTipText = (CultureInfo.CurrentCulture.Name == "zh-TW" ? "@UȮܦk" : "Next Click : Right Click");
                    notifyIcon1.ShowBalloonTip(10);
                    break;
                default:     //normal
                    this.notifyIcon1.BalloonTipText = (CultureInfo.CurrentCulture.Name == "zh-TW" ? "@U}" : "Click to Restore");
                    break;
            }
        }
        
        //add by gsyan
        void WiimoteMainForm_ErrorMessage(object sender, EventArgs<String> e)
        {
            BeginInvoke((MethodInvoker)delegate()
           {
               MessageBox.Show(e.Value);
           });
        }

        void ConnectionControl_StateChanged(object sender, EventArgs<ConnectionController.ConnectState> state)
        {
            switch (state.Value)
            {
                case ConnectionController.ConnectState.Connected:
                    controller.Enable();
                    pnlMainControls.Enabled = true;
                    break;
                case ConnectionController.ConnectState.NotFound:
                    controller.Disable();
                    pnlMainControls.Enabled = false;
                    break;
                case ConnectionController.ConnectState.Connecting:
                    controller.Disable();
                    pnlMainControls.Enabled = false;
                    break;
            }
        }

        void controller_StatusInfoChanged(object sender, EventArgs<WiimoteState> e)
        {
            if (Visible)
                BeginInvoke((MethodInvoker)delegate()
                {

                    WiimoteState state = e.Value;
                    if (pbBattery.Value != (int)state.Battery)
                    {
                        pbBattery.Value = (state.Battery>100 ? 100 : (int)state.Battery);   //modified by gsyan
                        lblBattery.Text = state.Battery.ToString("0\\%");
                    }
                    lblPoint1.Visible = state.IRState.IRSensors[0].Found;
                    lblPoint2.Visible = state.IRState.IRSensors[1].Found;
                    lblPoint3.Visible = state.IRState.IRSensors[2].Found;
                    lblPoint4.Visible = state.IRState.IRSensors[3].Found;

                    //--------------------------------------------------------
                    //show IR dots add by gsyan
                    PointF srcscaled = new PointF();
                    //TrackingLayout.Clear(Color.Gray);

                    UpdateTrackingUtilization(controller.SourceLayout);

                    Pen pointPen = new Pen(Color.Red);
                    for (int i = 0; i < 4; i++)
                    {
                        if (state.IRState.IRSensors[i].Found)
                        {
                            //srcscaled.X = state.IRState.IRSensors[i].Position.X * pbTrackingLayout.Width;
                            //srcscaled.Y = (1.0f-state.IRState.IRSensors[i].Position.Y) * pbTrackingLayout.Height;
                            srcscaled.X = state.IRState.IRSensors[i].RawPosition.X * pbTrackingLayout.Width / 1024;
                            srcscaled.Y = (768 - state.IRState.IRSensors[i].RawPosition.Y) * pbTrackingLayout.Height / 768;
                            switch (i)
                            {
                                case 0:
                                    pointPen.Color = Color.Red;
                                    break;
                                case 1:
                                    pointPen.Color = Color.Blue;
                                    break;
                                case 2:
                                    pointPen.Color = Color.Magenta;
                                    break;
                                case 3:
                                    pointPen.Color = Color.Green;
                                    break;
                            }
                            //TrackingLayout.DrawRectangle(pointPen, srcscaled.X - 10.0f, srcscaled.Y - 10.0f, 2.0f, 2.0f);
                            TrackingLayout.DrawEllipse(pointPen, srcscaled.X - 10.0f, srcscaled.Y - 10.0f, state.IRState.IRSensors[i].Size * 1.5f, state.IRState.IRSensors[i].Size * 1.5f);
                        }

                    }
                    pointPen.Dispose();
                    pbTrackingLayout.Invalidate();

                    //-------------------show IR dots end---------------------

                });
        }

        void CalibrationForm_Cancelled(object sender, EventArgs e)
        {
            controller.CalibrationCancel();
        }

        void controller_CalibrationStateChanged(object sender, EventArgs<Controller.CalibrationStateInfo> e)
        {
            if (e.Value.State > 0)
                BeginInvoke((MethodInvoker)delegate() { TopMost = false; CalibrationForm.ShowCalibration(e.Value.TargetPoint); });
            else
                BeginInvoke((MethodInvoker)delegate() { CalibrationForm.HideCalibration(); UpdateTrackingUtilization(controller.SourceLayout); TopMost = cbAlwaysOnTop.Checked; cbCursorControl.Checked = true; });  //add "cbCursorControl.Checked = true;" by gsyan
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            controller.CalibrationStateChanged -= controller_CalibrationStateChanged;
            controller.StatusInfoChanged -= controller_StatusInfoChanged;
            controller.Disable();
        }

        private void btnCalibrate_Click(object sender, EventArgs e)
        {
            controller.Calibrate();
        }

        private void cbCursorControl_CheckedChanged(object sender, EventArgs e)
        {
            controller.CursorEnabled = cbCursorControl.Checked;
        }

        void UpdateTrackingUtilization(PointF[] src)
        {
            PointF[] srcscaled = new PointF[4];
            for (int i = 0; i < srcscaled.Length; i++)
            {
                srcscaled[i] = new PointF();
                srcscaled[i].X = src[i].X * pbTrackingLayout.Width / 1024;
                srcscaled[i].Y = (768-src[i].Y) * pbTrackingLayout.Height / 768;        //modified by gsyan
            }
            //swap the last two points to get a proper rectangle
            PointF tmp = srcscaled[3];
            srcscaled[3] = srcscaled[2];
            srcscaled[2] = tmp;
            TrackingLayout.Clear(Color.Gray);
            TrackingLayout.DrawPolygon(Pens.Black, srcscaled);
            TrackingLayout.FillPolygon(Brushes.White, srcscaled);

            //area of ideal calibration coordinates (to match the screen)
            float idealArea = (1 - 2 * Controller.CALIBRATION_MARGIN) * 1024 * (1 - 2 * Controller.CALIBRATION_MARGIN) * 768;

            //area of quadrliatera
            float actualArea = 0.5f * Math.Abs((src[1].X - src[2].X) * (src[0].Y - src[3].Y) - (src[0].X - src[3].X) * (src[1].Y - src[2].Y));
            float util = (actualArea / idealArea) * 100;


            string value = util.ToString("0\\%");

            GraphicsPath pth = new GraphicsPath();
            pth.AddString(value, new FontFamily("Times New Roman"), 0, 20,
                new PointF(pbTrackingLayout.Width / 2 - 10, pbTrackingLayout.Height / 2 - 10), StringFormat.GenericTypographic);

            Pen p = new Pen(Color.White, 2.0f);
            TrackingLayout.FillPath(Brushes.Black, pth);
            p.Dispose();
            pth.Dispose();
            pbTrackingLayout.Invalidate();


        }

        private void tbSmoothingLevel_ValueChanged(object sender, EventArgs e)
        {
            controller.SmoothingAmount = tbSmoothingLevel.Value;
        }

        private void cbAlwaysOnTop_CheckedChanged(object sender, EventArgs e)
        {
            this.TopMost = cbAlwaysOnTop.Checked;            
        }

        private void cbMoveCursor_CheckedChanged(object sender, EventArgs e)
        {
            controller.MoveCursor = cbMoveCursor.Checked;
        }

        
        private void WiimoteMainForm_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == WindowState)
            {
                this.Hide();
                notifyIcon1.ShowBalloonTip(10);
                WindowState = FormWindowState.Minimized;
            }
        }
        private void minimizeToolStripMenuItem_Click(object sender, EventArgs e)
        {

            this.Hide();
            notifyIcon1.ShowBalloonTip(10);
            WindowState = FormWindowState.Minimized;
        }

        private void notifyIcon1_MouseClick(object sender, MouseEventArgs e)
        {
             this.Show();
             //notifyIcon1.ShowBalloonTip(1000);
              WindowState = FormWindowState.Normal;
        }

    }
}

